package org.eclipse.fdt.internal.core.f95parser;

import java.util.LinkedList;
import java.util.ListIterator;

public class ParseTreeNode extends AbstractParseTreeNode
{
    public ParseTreeNode(Nonterminal root)
    {
        this.root = root;
        this.children = new LinkedList();
    }

    public void addChild(AbstractParseTreeNode child)
    {
        if (child == null)
            return;
        else
        {
            children.add(child);
            child.parent = this;
        }
    }

    // -------------------------------------------------------------------------------------------

    private Nonterminal root;

    public Nonterminal getRootNonterminal()
    {
        return root;
    }

    private LinkedList children;

    public LinkedList getChildren()
    {
        return children;
    }

    // -------------------------------------------------------------------------------------------

    public void visitUsing(ParseTreeVisitor visitor)
    {
        root.visitParseTreeNodeUsing(this, visitor);

        visitor.preparingToVisitChildrenOf(this);
        for (ListIterator it = children.listIterator(); it.hasNext();)
            ((AbstractParseTreeNode)(it.next())).visitUsing(visitor);
        visitor.doneVisitingChildrenOf(this);
    }

    public void visitUsing(GenericParseTreeVisitor visitor)
    {
        visitor.visitParseTreeNode(this);

        visitor.preparingToVisitChildrenOf(this);
        for (ListIterator it = children.listIterator(); it.hasNext();)
            ((AbstractParseTreeNode)(it.next())).visitUsing(visitor);
        visitor.doneVisitingChildrenOf(this);
    }

    // -------------------------------------------------------------------------------------------
    
    private static final int INDENT_SIZE = 4;
    
    public String toString()
    {
        return toString(0);
    }
    
    public String toString(int numSpaces)
    {
        StringBuffer sb = new StringBuffer();
        sb.append(indent(numSpaces));
        sb.append(root.getDescription());
        sb.append("\n");

        for (ListIterator it = children.listIterator(); it.hasNext();)
            sb.append(((AbstractParseTreeNode)(it.next())).toString(numSpaces + INDENT_SIZE));
        
        return sb.toString();
    }
}
